@@ -2,8 +2,8 @@
 #
 # argmax.geno.R
 #
-# copyright (c) 2001, Karl W Broman, Johns Hopkins University
-# last modified Sept, 2001
+# copyright (c) 2001-2, Karl W Broman, Johns Hopkins University
+# last modified June, 2002
 # first written Nov, 2001
 # Licensed under the GNU General Public License version 2 (June, 1991)
 # 
@@ -21,20 +21,26 @@
 
 argmax.geno <-
 function(cross, step=0, off.end=0, error.prob=0,
-         map.function=c("haldane","kosambi","c-f"))
+         map.function=c("haldane","kosambi","c-f","morgan"))
 {
   # map function
   map.function <- match.arg(map.function)
   if(map.function=="kosambi") mf <- mf.k
   else if(map.function=="c-f") mf <- mf.cf
+  else if(map.function=="morgan") mf <- mf.m
   else mf <- mf.h
 
-  # don't let error.prob be exactly zero, just in case
+  # don't let error.prob be exactly zero (or >1)
   if(error.prob < 1e-50) error.prob <- 1e-50
+  if(error.prob > 1) {
+    error.prob <- 1-1e-50
+    warning("error.prob shouldn't be > 1!")
+  }
 
   n.ind <- nind(cross)
   n.chr <- nchr(cross)
   n.mar <- nmar(cross)
+  type <- class(cross)[1]
 
   # loop over chromosomes
   for(i in 1:n.chr) {
@@ -42,29 +48,27 @@ function(cross, step=0, off.end=0, error.prob=0,
     else temp.offend <- off.end
 
     # which type of cross is this?
-    if(class(cross)[1] == "f2") {
+    if(type=="f2") {
       one.map <- TRUE
-      if(class(cross$geno[[i]]) == "A") { # autosomal
+      if(class(cross$geno[[i]]) == "A") # autosomal
         cfunc <- "argmax_geno_f2"
-      }
-      else {                             # X chromsome 
+      else                              # X chromsome 
         cfunc <- "argmax_geno_bc"
-      }
     }
-    else if(class(cross)[1] == "bc") {
+    else if(type=="bc" || type=="riself" || type=="risib") {
       cfunc <- "argmax_geno_bc"
       one.map <- TRUE
     }
-    else if(class(cross)[1] == "4way") {
+    else if(type == "4way") {
       cfunc <- "argmax_geno_4way"
       one.map <- FALSE
     }
-    else if(class(cross)[1] == "f2ss") {
+    else if(type == "f2ss") {
       cfunc <- "argmax_geno_f2ss"
       one.map <- FALSE
     }
     else stop(paste("argmax.geno not available for cross type",
-                    class(cross)[1], "."))
+                    type, "."))
 
     # genotype data
     gen <- cross$geno[[i]]$data
@@ -75,6 +79,8 @@ function(cross, step=0, off.end=0, error.prob=0,
       # recombination fractions
       map <- create.map(cross$geno[[i]]$map,step,temp.offend)
       rf <- mf(diff(map))
+      if(type=="risib" || type=="riself")
+        rf <- adjust.rf.ri(rf,substr(type,3,nchar(type)))
       rf[rf < 1e-14] <- 1e-14
 
       # new genotype matrix with pseudomarkers filled in
@@ -141,6 +147,10 @@ function(cross, step=0, off.end=0, error.prob=0,
     attr(cross$geno[[i]]$argmax,"off.end") <- temp.offend
   }
 
+  # store argmax values as integers
+  for(i in 1:nchr(cross))
+    storage.mode(cross$geno[[i]]$argmax) <- "integer"
+
   cross
 }
 
